from rest_framework.test import APIClient

from tests.test_base import TestBase


class TestProfileViewSet(TestBase):
    def setUp(self):
        super().setUp()
        self.api_client = APIClient()

    def test_register_password_is_to_simillar_and_short(self):
        user = {
            "password": "string",
            "password_confirm": "string",
            "username": "string",
            "first_name": "string",
            "last_name": "string",
            "email": "user@example.com",
        }
        response = self.api_client.post("/account/register/", user, format="json")
        expected_response = (
            b'{"password":["The password is too similar to the username.","This password is too '
            b'short. It must contain at least 8 characters."],"non_field_errors":[]}'
        )
        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.content, expected_response)

    def test_register_password_is_to_short(self):
        user = {
            "password": "ad124sD",
            "password_confirm": "ad124sD",
            "username": "string",
            "first_name": "string",
            "last_name": "string",
            "email": "user@example.com",
        }
        response = self.api_client.post("/account/register/", user, format="json")
        expected_response = (
            b'{"password":["This password is too short. It must contain at least 8 characters."],'
            b'"non_field_errors":[]}'
        )
        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.content, expected_response)

    def test_register_password_is_to_similar(self):
        user = {
            "password": "string12",
            "password_confirm": "string12",
            "username": "string",
            "first_name": "string",
            "last_name": "string",
            "email": "user@example.com",
        }
        response = self.api_client.post("/account/register/", user, format="json")
        expected_response = b'{"password":["The password is too similar to the username."],"non_field_errors":[]}'
        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.content, expected_response)

    def test_register_success(self):
        user = {
            "password": "asd124sD",
            "password_confirm": "asd124sD",
            "username": "string",
            "first_name": "string",
            "last_name": "string",
            "email": "user@example.com",
        }
        response = self.api_client.post("/account/register/", user, format="json")
        expected_response = (
            b'{"id":3,"username":"string","first_name":"string","last_name":"string",' b'"email":"user@example.com"}'
        )
        self.assertEqual(response.status_code, 201)
        self.assertEqual(response.content, expected_response)

    def test_register_success_duplicate(self):
        user = {
            "password": "asd124sD",
            "password_confirm": "asd124sD",
            "username": "string",
            "first_name": "string",
            "last_name": "string",
            "email": "user@example.com",
        }
        response = self.api_client.post("/account/register/", user, format="json")
        expected_response = (
            b'{"id":3,"username":"string","first_name":"string","last_name":"string",' b'"email":"user@example.com"}'
        )
        self.assertEqual(response.status_code, 201)
        self.assertEqual(response.content, expected_response)

        response = self.api_client.post("/account/register/", user, format="json")
        expected_response = b'{"username":["A user with that username already exists."]}'
        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.content, expected_response)



# Generated by Django 4.2.1 on 2023-08-20 17:06

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("demo_django_base", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="project",
            name="slug",
            field=models.SlugField(max_length=80, unique=True, verbose_name="Slug"),
        ),
    ]



from django.db import models
from django.db.models import fields
from taggit.managers import TaggableManager

from django_project_base.auth.models import BasePermission, BaseRole
from django_project_base.base.models import (
    BaseInvite,
    BaseMergeUserGroup,
    BaseProfile,
    BaseProject,
    BaseProjectMember,
    BaseProjectSettings,
    BaseTag,
    DpbTaggedItemThrough,
)


class UserProfile(BaseProfile):
    pass


class Project(BaseProject):
    owner = models.ForeignKey(UserProfile, related_name="user_profiles", on_delete=models.CASCADE)


class ProjectMember(BaseProjectMember):
    pass


class DemoProjectTag(BaseTag):
    content = models.CharField(max_length=20, null=True, blank=True)

    class Meta(BaseTag.Meta):
        verbose_name = "Tag"
        verbose_name_plural = "Tags"


class DpbTaggedItemThroughDemo(DpbTaggedItemThrough):
    pass


class Apartment(models.Model):
    number = fields.IntegerField()
    tags = TaggableManager(through=DpbTaggedItemThroughDemo, related_name="apartment_tags")


class MergeUserGroup(BaseMergeUserGroup):
    pass


class Role(BaseRole):
    pass


class Permission(BasePermission):
    pass


class ProjectSettings(BaseProjectSettings):
    pass


class ProjectInvite(BaseInvite):
    pass



from collections.abc import Iterable
from typing import Any


def parse_settings(input_settings: tuple) -> None:
    from django.conf import settings

    for _setting in input_settings:
        setting_name: str = _setting["name"]
        setting_default_value: Any = _setting["default"]
        if hasattr(settings, setting_name):
            _setting_existing_val: Any = getattr(settings, setting_name)
            if isinstance(_setting_existing_val, (tuple, list)):
                _setting_existing_val: list = list(_setting_existing_val)
                if isinstance(setting_default_value, Iterable):
                    for v in setting_default_value:
                        _setting_existing_val.append(v)
                else:
                    _setting_existing_val.append(setting_default_value)
                _setting_existing_val: tuple = tuple(dict.fromkeys(_setting_existing_val))
                setattr(settings, setting_name, _setting_existing_val)
            elif isinstance(_setting_existing_val, dict):
                _setting = setting_default_value
                _setting.update(_setting_existing_val)
                setattr(settings, setting_name, _setting)
            else:
                pass
        else:
            setattr(settings, setting_name, setting_default_value)



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("demo_django_base", "0008_projectinvite"),
    ]

    operations = [
        migrations.AddField(
            model_name="projectsettings",
            name="pending_value",
            field=models.CharField(max_length=320, null=True, verbose_name="Pending value"),
        ),
        migrations.AddField(
            model_name="projectsettings",
            name="action_required",
            field=models.BooleanField(default=False, verbose_name="Action required", null=True, blank=True),
        ),
    ]



# Generated by Django 4.2.4 on 2023-08-29 12:23

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("demo_django_base", "0002_alter_project_slug"),
    ]

    operations = [
        migrations.AddField(
            model_name="project",
            name="email_sender_id",
            field=models.CharField(max_length=320, null=True),
        ),
        migrations.AddField(
            model_name="project",
            name="sms_sender_id",
            field=models.CharField(max_length=11, null=True),
        ),
    ]



# Generated by Django 4.2.4 on 2023-10-26 06:52

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("demo_django_base", "0009_projectsettings_pending_value"),
    ]

    operations = [
        migrations.AlterField(
            model_name="projectsettings",
            name="action_required",
            field=models.BooleanField(blank=True, default=False, null=True),
        ),
        migrations.AlterField(
            model_name="projectsettings",
            name="pending_value",
            field=models.CharField(blank=True, max_length=320, null=True, verbose_name="Pending value"),
        ),
    ]



#!/usr/bin/env python
"""Django's command-line utility for administrative tasks."""

import os
import sys


def main():
    """Run administrative tasks."""
    os.environ.setdefault("DJANGO_SETTINGS_MODULE", "example.setup.settings")
    try:
        from django.core.management import execute_from_command_line
    except ImportError as exc:
        raise ImportError(
            "Couldn't import Django. Are you sure it's installed and "
            "available on your PYTHONPATH environment variable? Did you "
            "forget to activate a virtual environment?"
        ) from exc

    params = list(sys.argv)
    if params[1] == "makemessages":
        params.extend(("-i", "node_modules", "-i", "static", "-i", "coverage"))
        params.extend(("-i", "tests", "-i", "setup", "-i", "examples", "-i", "dist"))
        print("Modified makemessages: will process both django and djangojs domains")
        execute_from_command_line(params + ["-d", "djangojs", "-e", "js,ts,vue"])

    execute_from_command_line(sys.argv)


if __name__ == "__main__":
    main()



# Generated by Django 3.2.25 on 2025-01-09 10:04

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('notifications', '0010_djangoprojectbasenotification_extra_data'),
    ]

    operations = [
        migrations.CreateModel(
            name='DjangoProjectBaseMessageAttachment',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, primary_key=True, serialize=False, verbose_name='Id')),
                ('file', models.FileField(upload_to='', verbose_name='Attachment')),
                ('message', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='notifications.djangoprojectbasemessage', verbose_name='Message')),
            ],
            options={
                'verbose_name': 'MessageAttachment',
                'abstract': False,
            },
        ),
    ]



import swapper
from django.utils.crypto import get_random_string
from rest_framework.test import APIClient

from django_project_base.settings import TEST_USER_ONE_DATA
from example.demo_django_base.models import UserProfile
from tests.test_base import TestBase


class TestProjectSettings(TestBase):
    url: str = "/project-settings"

    def setUp(self):
        super().setUp()
        self.settingsModel = swapper.load_model("django_project_base", "ProjectSettings")
        self.api_client = APIClient()
        self._login_with_test_user_one()
        self.project = swapper.load_model("django_project_base", "Project").objects.create(
            **{
                "name": "test-project",
                "owner": UserProfile.objects.get(username=TEST_USER_ONE_DATA["username"]),
                "slug": get_random_string(length=8),
            }
        )
        self._create_fixtures()

    def _create_fixtures(self):
        self.settingsModel.objects.create(
            **{
                "name": "test",
                "description": "test",
                "value": "test",
                "value_type": self.settingsModel.VALUE_TYPE_CHAR,
                "project": self.project,
            }
        )

    def test_list_settings(self):
        self.assertEqual(1, len(self.api_client.get(self.url, HTTP_CURRENT_PROJECT=self.project.slug).data))



import threading
import time

from django.core.cache import cache, caches
from django.test import override_settings, SimpleTestCase

from django_project_base.caching import CacheCounter
from django_project_base.caching.cache_queue import CacheQueue


def get_redis_cache_backend_name():
    if CacheQueue.is_redis_cache_backend("default"):
        return "django_redis.cache.RedisCache"
    else:
        return "django.core.cache.backends.locmem.LocMemCache"


class TestCaching(SimpleTestCase):
    @override_settings(
        CACHES={
            "default": {
                "BACKEND": get_redis_cache_backend_name(),
                "LOCATION": "redis://127.0.0.1:6379?db=1",
                "OPTIONS": {
                    "CLIENT_CLASS": "django_redis.client.DefaultClient",
                },
            },
        }
    )
    def test_cache_counter_redis_cache(self):
        # Calling cache counter test for redis cache backend
        self._test_cache_counter()

    @override_settings(
        CACHES={
            "default": {
                "BACKEND": "django.core.cache.backends.locmem.LocMemCache",
                "LOCATION": "",
            }
        }
    )
    def test_cache_counter_loc_mem_cache(self):
        # Calling cache counter test for locmem cache backend
        self._test_cache_counter()

    def _test_cache_counter(self):
        caches["default"].clear()

        # Increasing counters
        self._change_counters(True)

        for c in range(3):
            self.assertEqual(280, cache.get(f"cnt_{c + 1}", 0))

        # Decreasing counters
        self._change_counters(False)

        for c in range(3):
            self.assertEqual(0, cache.get(f"cnt_{c + 1}", 0))

        caches["default"].clear()
        # Increasing counters with start at 10
        self._change_counters(True, 10)
        for c in range(3):
            self.assertEqual(290, cache.get(f"cnt_{c + 1}", 0))

        # Checking if counter only lasts till timeout
        cc = CacheCounter("timeout", timeout=2)
        start = time.time()
        cc.incr()
        while cache.get("timeout", 0) and time.time() - start < 4:
            time.sleep(0.1)
        duration = time.time() - start
        self.assertTrue(2 < duration < 3)

        increased = False
        start = time.time()
        cc.incr()
        # Checking if counter only lasts till timeout after last change
        while cache.get("timeout", 0) and time.time() - start < 5:
            if not increased and time.time() - start > 1:
                cc.incr()
                increased = True
            time.sleep(0.1)
        duration = time.time() - start
        self.assertTrue(3 < duration < 4)

    # noinspection PyMethodMayBeStatic
    def _change_counters(self, increase=True, start=0):
        def _increase_counter(key, value):
            from django.db import connection

            counter = CacheCounter(key)
            for _i in range(10):
                params = dict()
                if value != 1:
                    params["step"] = value
                if start != 0:
                    params["start"] = start
                counter.incr(**params)
                time.sleep(0.001)
            connection.close()

        # I maintain 3 counters. Each is getting updated by 7 threads (at same time).
        # Each thread is changing counters with different step
        # So checking if all updates are taken into account.

        test_processes = []
        for i in range(7):
            for c in range(3):
                test_processes.append(
                    threading.Thread(target=_increase_counter, args=(f"cnt_{c + 1}", (i + 1) * (1 if increase else -1)))
                )
        for process in test_processes:
            process.start()

        for process in test_processes:
            process.join()

    @override_settings(
        CACHES={
            "default": {
                "BACKEND": get_redis_cache_backend_name(),
                "LOCATION": f"redis://127.0.0.1:6379?db=1",
                "OPTIONS": {
                    "CLIENT_CLASS": "django_redis.client.DefaultClient",
                },
            },
        }
    )
    def test_cache_queue_redis_cache(self):
        # Calling cache queue test for redis cache backend
        self._test_cache_queue()

    @override_settings(
        CACHES={
            "default": {
                "BACKEND": "django.core.cache.backends.locmem.LocMemCache",
                "LOCATION": "",
            }
        }
    )
    def test_cache_queue_loc_mem_cache(self):
        # Calling cache queue test for locmem cache backend
        self._test_cache_queue()

    def _test_cache_queue(self):
        caches["default"].clear()
        cache_queue = CacheQueue.get_cache_queue("test", timeout=None)

        # inserting data to end of queue
        cache_queue.rpush("1")
        cache_queue.rpush("2", "3", "4")
        values = ["5", "6", "7"]
        cache_queue.rpush(*values)
        whole_list = cache_queue.lrange()
        self.assertTrue(isinstance(whole_list, list))
        self.assertEqual(len(whole_list), 7)
        for i in range(7):
            self.assertEqual(whole_list[i].decode("utf-8"), str(i + 1))

        # poping data from start of queue
        first_item = cache_queue.lpop()
        # If only one item is retrieved it should not be in list
        self.assertFalse(isinstance(first_item, list))
        self.assertEqual(first_item.decode("utf-8"), "1")
        next_items = cache_queue.lpop(3)
        # If more than item is retrieved it should be in list
        self.assertTrue(isinstance(next_items, list))
        self.assertEqual(len(next_items), 3)
        for i in range(3):
            self.assertEqual(next_items[i].decode("utf-8"), str(i + 2))

        next_items = cache_queue.lpop(30)
        self.assertTrue(isinstance(next_items, list))
        self.assertEqual(len(next_items), 3)
        for i in range(3):
            self.assertEqual(next_items[i].decode("utf-8"), str(i + 5))

        next_items = cache_queue.lpop(30)
        # If there is no items in queue, pop should return None
        self.assertIsNone(next_items)

        whole_list = cache_queue.lrange()
        self.assertTrue(isinstance(whole_list, list))
        self.assertEqual(len(whole_list), 0)

        cache_queue = CacheQueue.get_cache_queue("test1")
        # inserting data to start of queue
        cache_queue.lpush("1")
        cache_queue.lpush("2", "3", "4")
        values = ["5", "6", "7"]
        cache_queue.lpush(*values)
        whole_list = cache_queue.lrange()
        self.assertTrue(isinstance(whole_list, list))
        self.assertEqual(len(whole_list), 7)
        for i in range(7):
            self.assertEqual(whole_list[i].decode("utf-8"), str(7 - i))

        # poping data from end of queue
        last_item = cache_queue.rpop()
        # If only one item is retrieved it should not be in list
        self.assertFalse(isinstance(last_item, list))
        self.assertEqual(last_item.decode("utf-8"), "1")
        prev_items = cache_queue.rpop(3)
        # If more than item is retrieved it should be in list
        self.assertTrue(isinstance(prev_items, list))
        self.assertEqual(len(prev_items), 3)
        for i in range(3):
            self.assertEqual(prev_items[i].decode("utf-8"), str(i + 2))

        prev_items = cache_queue.rpop(30)
        self.assertTrue(isinstance(prev_items, list))
        self.assertEqual(len(prev_items), 3)
        for i in range(3):
            self.assertEqual(prev_items[i].decode("utf-8"), str(i + 5))

        prev_items = cache_queue.rpop(30)
        # If there is no items in queue, pop should return None
        self.assertIsNone(prev_items)

        whole_list = cache_queue.lrange()
        self.assertTrue(isinstance(whole_list, list))
        self.assertEqual(len(whole_list), 0)

        cache_queue.set_timeout(2)
        cache_queue.rpush(b"1", b"2", b"3", b"4")
        # retrieving, but not removing items from start of queue
        next_items = cache_queue.lrange(2)
        self.assertTrue(isinstance(next_items, list))
        self.assertEqual(len(next_items), 3)
        for i in range(3):
            self.assertEqual(next_items[i], str(i + 1).encode("utf-8"))
        # removing items from start of queue
        cache_queue.ltrim(len(next_items))

        start = time.time()
        self.assertEqual(len(cache_queue.lrange()), 1)
        # Checking if queue only persists until timeout
        while cache_queue.lrange() and time.time() - start < 4:
            time.sleep(0.1)
        duration = time.time() - start
        self.assertTrue(2 < duration < 3)



from unittest.mock import patch

from django.test import TestCase

from django_project_base.celery.background_tasks.notification_tasks import SendNotificationTask


class SendNotificationTaskTest(TestCase):

    def setUp(self):
        self.task = SendNotificationTask()

    @patch('django_project_base.notifications.base.send_notification_service.SendNotificationService.make_send')
    @patch('django_project_base.celery.background_tasks.notification_tasks.SendNotificationTask.apply_async')
    def test_send_notification_task(self, mock_apply_async, mock_make_send):
        mock_make_send.result = None
        self.task.apply((None, None))



import swapper

from rest_framework import status
from rest_framework.test import APIClient

from django_project_base.base.auth_backends import user_cache_invalidate
from example.demo_django_base.models import UserProfile
from tests.test_base import TestBase


class TestProfileViewSet(TestBase):
    def setUp(self):
        super().setUp()
        self.api_client = APIClient()

    def test_get_current_profile(self):
        self.assertTrue(self.api_client.login(username="miha", password="mihamiha"), "Not logged in")

        # Če ni izbranega projekta, potem vrne 403
        response = self.api_client.get("/account/profile", {}, format="json")
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

        # Za current profile pa ni potrebnega projekta
        response = self.api_client.get("/account/profile/current", {}, format="json")
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data["full_name"], "Miha Novak")

        slug = swapper.load_model("django_project_base", "Project").objects.first().slug

        response = self.api_client.get("/account/profile", {}, format="json", HTTP_CURRENT_PROJECT=slug)
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_insert_profile(self):
        self.assertTrue(self.api_client.login(username="miha", password="mihamiha"), "Not logged in")
        profile = {
            "password": "string",
            "last_login": "2021-06-02T08:49:13.618Z",
            "is_superuser": False,
            "username": "string",
            "first_name": "string",
            "last_name": "string",
            "email": "user@example.com",
            "is_staff": False,
            "is_active": True,
            "date_joined": "2021-06-02T08:49:13.618Z",
            "bio": "string",
            "phone_number": "string",
            "language": "string",
            "theme": "string",
            "avatar": "string",
            "reverse_full_name_order": False,
            "groups": None,
            "user_permissions": None,
        }
        slug = swapper.load_model("django_project_base", "Project").objects.first().slug

        response = self.api_client.post("/account/profile", profile, format="json", HTTP_CURRENT_PROJECT=slug)
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

    def test_update_profile(self):
        import swapper

        from django_project_base.rest.project import ProjectSerializer

        self.assertTrue(self.api_client.login(username="miha", password="mihamiha"), "Not logged in")
        project = ProjectSerializer.Meta.model.objects.last()
        swapper.load_model("django_project_base", "ProjectMember").objects.create(project_id=project.id, member_id=1)

        response = self.api_client.patch(
            "/account/profile/1",
            {"bio": "Sample bio text.", "avatar": None},
            format="json",
            HTTP_CURRENT_PROJECT=project.slug,
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        response = self.api_client.get("/account/profile/1", {}, format="json", HTTP_CURRENT_PROJECT=project.slug)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data.get("bio", False), "Sample bio text.")

    def test_search_url_is_disabled(self):
        self.assertTrue(self.api_client.login(username="miha", password="mihamiha"), "Not logged in")
        response = self.api_client.get("/account/profile/search/miha", {}, format="json")
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

    def test_search_query(self):
        import swapper

        from django_project_base.rest.project import ProjectSerializer

        self.assertTrue(self.api_client.login(username="miha", password="mihamiha"), "Not logged in")
        project = ProjectSerializer.Meta.model.objects.last()
        swapper.load_model("django_project_base", "ProjectMember").objects.create(project_id=project.id, member_id=1)

        response = self.api_client.get(
            "/account/profile?search=mi", {}, format="json", HTTP_CURRENT_PROJECT=project.slug
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data), 1)
        self.assertEqual(response.data[0]["full_name"], "Miha Novak")

        # Change with afca5415 - "only show users of current project or none at all" 22.12.2023
        # we cannot query other users if we are not admins
        # response = self.api_client.get(
        #     "/account/profile?search=j",
        #     {},
        #     format="json",
        #     HTTP_CURRENT_PROJECT=project.slug
        # )
        # self.assertEqual(response.status_code, status.HTTP_200_OK)
        # self.assertEqual(len(response.data), 0)
        #
        # miha = UserProfile.objects.get(username="miha")
        # miha.is_staff = True
        # miha.save()
        # user_cache_invalidate(miha)
        # response = self.api_client.get(
        #     "/account/profile?search=j",
        #     {},
        #     format="json",
        #     HTTP_CURRENT_PROJECT=project.slug
        # )
        #
        # self.assertEqual(response.status_code, status.HTTP_200_OK)
        # self.assertEqual(len(response.data), 1)
        # self.assertEqual(response.data[0]["full_name"], "Janez Novak")

    def test_supperss_is_staff_is_superuser(self):
        import swapper

        from django_project_base.rest.project import ProjectSerializer

        self.assertTrue(self.api_client.login(username="miha", password="mihamiha"), "Not logged in")
        project = ProjectSerializer.Meta.model.objects.last()
        swapper.load_model("django_project_base", "ProjectMember").objects.create(project_id=project.id, member_id=1)
        swapper.load_model("django_project_base", "ProjectMember").objects.create(project_id=project.id, member_id=2)

        response = self.api_client.get("/account/profile/1", {}, format="json", HTTP_CURRENT_PROJECT=project.slug)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data.get("is_staff", "not_exist"), "not_exist")
        self.assertEqual(response.data.get("is_superuser", "not_exist"), "not_exist")

        miha = UserProfile.objects.get(username="miha")
        miha.is_staff = True
        miha.is_superuser = True
        miha.save()
        user_cache_invalidate(miha)

        response = self.api_client.get("/account/profile/1", {}, format="json", HTTP_CURRENT_PROJECT=project.slug)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data.get("is_staff", "not_exist"), True)
        self.assertEqual(response.data.get("is_superuser", "not_exist"), True)

        response = self.api_client.get("/account/profile/2", {}, format="json", HTTP_CURRENT_PROJECT=project.slug)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data.get("is_staff", "not_exist"), False)
        self.assertEqual(response.data.get("is_superuser", "not_exist"), False)

    def test_profile_destroy(self):
        import swapper

        from django_project_base.rest.project import ProjectSerializer

        self.assertTrue(self.api_client.login(username="miha", password="mihamiha"), "Not logged in")
        project = ProjectSerializer.Meta.model.objects.last()
        swapper.load_model("django_project_base", "ProjectMember").objects.create(project_id=project.id, member_id=2)
        response = self.api_client.delete("/account/profile/1", {}, format="json", HTTP_CURRENT_PROJECT=project.slug)
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
        expected_response = b'{"detail":"You do not have permission to perform this action."}'
        self.assertEqual(response.content, expected_response)

        miha = UserProfile.objects.get(username="miha")
        miha.is_staff = True
        miha.is_superuser = True
        miha.save()
        user_cache_invalidate(miha)
        response = self.api_client.delete("/account/profile/2", {}, format="json", HTTP_CURRENT_PROJECT=project.slug)
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

    def test_profile_destroy_my_profile(self):
        self.assertTrue(self.api_client.login(username="miha", password="mihamiha"), "Not logged in")
        response = self.api_client.get("/account/profile/current", {}, format="json")
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        response = self.api_client.delete("/account/profile/current", {}, format="json")
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)
        response = self.api_client.get("/account/profile/current", {}, format="json")
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)



import threading
import time

from django.core.cache import cache, caches
from django.test import override_settings, SimpleTestCase

from django_project_base.caching import CacheCounter
from django_project_base.serialization import CacheLock, NoTimeoutCheck, ObjectLockTimeout


class TestSerialization(SimpleTestCase):

    @override_settings(CACHES={
        'default': {
            'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
            'LOCATION': '',
        }
    })
    def test_serialization_blocking(self):
        times = dict()
        caches["default"].clear()

        def _run_process(name):
            from django.db import connection
            nonlocal times
            times.setdefault(name, dict())
            with CacheLock("Process"):
                times[name]["start"] = time.time()
                CacheCounter("processed").incr()
                time.sleep(1)
                times[name]["end"] = time.time()
            connection.close()

        # Spawning 3 processes that tries to execute CacheLock code at the same time.
        # They should wait for each other (but order is not guaranteed).
        # I am checking if execution times of threads are not overlapping
        processes = []
        for i in range(3):
            processes.append(threading.Thread(target=_run_process, args=(f"Thread-{i + 1}",)))
        for p in processes:
            p.start()
            time.sleep(.1)
        for p in processes:
            p.join()

        self.assertEqual(cache.get("processed"), 3)
        end = 0
        for key in sorted(times.keys(), key=lambda k: times[k]["start"]):
            start = times[key]["start"]
            self.assertLessEqual(end, start)
            end = times[key]["end"]

    @override_settings(CACHES={
        'default': {
            'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
            'LOCATION': '',
        }
    })
    def test_serialization_blocking_timeout(self):

        caches["default"].clear()
        expect_processed = 0
        expect_cl_timeout = 0
        expect_to_check = 0
        expect_invalid_to = 0

        def _check_counters():
            self.assertEqual(cache.get("processed", 0), expect_processed)
            self.assertEqual(cache.get("cl_timeout", 0), expect_cl_timeout)
            self.assertEqual(cache.get("to_check", 0), expect_to_check)
            self.assertEqual(cache.get("invalid_to", 0), expect_invalid_to)

        def _run_process(timeout_check, sleep, timeout):
            from django.db import connection
            try:
                with CacheLock("Process", timeout=timeout,
                               silence_object_lock_timeout=True if timeout == -1 else False) as cl:
                    if timeout_check:
                        cl()
                    CacheCounter("processed").incr()
                    if sleep:
                        time.sleep(2)
            except ObjectLockTimeout:
                CacheCounter("cl_timeout").incr()
            except NoTimeoutCheck:
                CacheCounter("to_check").incr()
            except Exception:
                CacheCounter("invalid_to").incr()
            connection.close()

        # Invalid timeout. CacheLock should raise Exception
        p = threading.Thread(target=_run_process, args=(False, False, -2))
        p.start()
        p.join()
        expect_invalid_to += 1
        _check_counters()

        # No timeout check. CacheLock should raise NoTimeoutCheck exception.
        # But this only happens on exit. So with block is processed
        p = threading.Thread(target=_run_process, args=(False, False, 1))
        p.start()
        p.join()

        expect_processed += 1
        expect_to_check += 1
        _check_counters()

        # Timeout greater than function execution time. Only first CacheLock call should be processed.
        # There should be two ObjectLockTimeout exceptions raised
        processes = []
        for i in range(3):
            processes.append(threading.Thread(target=_run_process, args=(True, True, 1)))
        for p in processes:
            p.start()
            time.sleep(.1)
        for p in processes:
            p.join()

        expect_processed += 1
        expect_cl_timeout += 2
        _check_counters()

        # Just checking if there is no lock anymore. So this call should be processed without any issues
        p = threading.Thread(target=_run_process, args=(True, False, 1))
        p.start()
        p.join()

        expect_processed += 1
        _check_counters()

        # Timeout set to -1, which means that if with block is currently executing other thread should just skip it.
        # So in this case with block should be processed only once.
        processes = []
        for i in range(3):
            processes.append(threading.Thread(target=_run_process, args=(True, True, -1)))
        for p in processes:
            p.start()
            time.sleep(.1)
        for p in processes:
            p.join()

        expect_processed += 1
        _check_counters()



import datetime

from rest_framework import status
from rest_framework.response import Response
from rest_framework.test import APIClient

from django_project_base.notifications.models import DjangoProjectBaseMessage
from django_project_base.utils import get_pk_name
from tests.test_base import TestBase


class TestMaintenanceNotifications(TestBase):
    url = "/maintenance-notification/"

    def setUp(self):
        super().setUp()
        self.api_client = APIClient()

    def _create_maintenance_notification(self, payload: dict) -> Response:
        self._login_with_test_user_one()
        _payload: dict = {
            "delayed_to": int((datetime.datetime.now() + datetime.timedelta(hours=1)).timestamp()),
            "message": {"body": "Planned maintenance"},
        }
        _payload.update(payload)
        return self.api_client.post(self.url, _payload, format="json")

    def test_create_maintenance_notification(self):
        self.assertEqual(status.HTTP_201_CREATED, self._create_maintenance_notification({}).status_code)
        self.assertEqual(
            self._create_maintenance_notification(dict(delayed_to=datetime.datetime.now())).status_code,
            status.HTTP_400_BAD_REQUEST,
        )
        self.assertEqual(
            self._create_maintenance_notification(
                dict(delayed_to=(datetime.datetime.now() - datetime.timedelta(hours=1)).timestamp())
            ).status_code,
            status.HTTP_400_BAD_REQUEST,
        )

    def test_list_maintenance_notification(self):
        create_response: Response = self._create_maintenance_notification({})
        self.assertEqual(status.HTTP_201_CREATED, create_response.status_code)
        list_response: Response = self.api_client.get(self.url)
        self.assertEqual(1, len(list_response.data))

        # acknowledge notification - it was read by user
        acknowledged_response: Response = self.api_client.post(
            f"{self.url}acknowledged/",
            {
                get_pk_name(DjangoProjectBaseMessage): list_response.data[0][get_pk_name(DjangoProjectBaseMessage)],
                "acknowledged_identifier": 5,
            },
        )
        self.assertEqual(status.HTTP_201_CREATED, acknowledged_response.status_code)
        self.assertEqual(1, len(self.api_client.get(self.url).data))



import os

from rest_framework import status
from rest_framework.test import APIClient

from django_project_base.base.auth_backends import user_cache_invalidate
from example.demo_django_base.models import UserProfile
from tests.test_base import TestBase


class TestLoginViewset(TestBase):
    url_prefix = "/account/"

    def setUp(self):
        super().setUp()
        self.api_client = APIClient()

    def test_login_no_post_data(self):
        response = self.api_client.post(os.path.join(self.url_prefix, "login"), {}, format="json")
        self.assertEqual(response.status_code, 400)
        self.assertEqual(
            response.content, b'{"login":["This field is required."],"password":["This field is required."]}'
        )

    def test_login_no_post_login(self):
        response = self.api_client.post(os.path.join(self.url_prefix, "login"), {"password": "mihamiha"}, format="json")
        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.content, b'{"login":["This field is required."]}')

    def test_login_no_post_password(self):
        response = self.api_client.post(os.path.join(self.url_prefix, "login"), {"login": "miha"}, format="json")
        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.content, b'{"password":["This field is required."]}')

    def test_login_wrong_user(self):
        response = self.api_client.post(
            os.path.join(self.url_prefix, "login"), {"login": "mihamiha", "password": "mihamiha"}, format="json"
        )
        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.content, b'{"detail":"Login or password invalid."}')

    def test_login_wrong_password(self):
        response = self.api_client.post(
            os.path.join(self.url_prefix, "login"), {"login": "miha", "password": "miha"}, format="json"
        )
        self.assertEqual(response.status_code, 400)
        self.assertEqual(response.content, b'{"detail":"Login or password invalid."}')

    def test_login(self):
        response = self.api_client.post(
            os.path.join(self.url_prefix, "login"), {"login": "miha", "password": "mihamiha"}, format="json"
        )
        self.assertIsNotNone(response.cookies.get("sessionid", None))
        self.assertIsNotNone(response.cookies.get("csrftoken", None))
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_login_json(self):
        # make Miha a superuser
        miha = UserProfile.objects.get(username="miha")
        miha.is_staff = True
        miha.is_superuser = True
        miha.save()
        user_cache_invalidate(miha)

        response = self.api_client.post(
            os.path.join(self.url_prefix, "login"),
            {"login": "miha", "password": "mihamiha", "return-type": "json"},
            format="json",
        )
        self.assertIsNone(response.cookies.get("sessionid", None))
        self.assertIsNone(response.cookies.get("csrftoken", None))
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.api_client.credentials(HTTP_AUTHORIZATION="sessionid " + response.data.get("sessionid", None))
        response = self.api_client.get(
            os.path.join(self.url_prefix, "/account/profile/current"), {"return-type": "json"}, format="json"
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        response = self.api_client.put("/account/impersonate", {"username": "janez"}, format="json")
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_login_json_missing_session(self):
        response = self.api_client.post(
            os.path.join(self.url_prefix, "login"),
            {"login": "miha", "password": "mihamiha", "return-type": "json"},
            format="json",
        )
        self.assertIsNone(response.cookies.get("sessionid", None))
        self.assertIsNone(response.cookies.get("csrftoken", None))
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        response = self.api_client.get(os.path.join(self.url_prefix, "/account/profile/current"), {}, format="json")
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
        self.assertEqual(response.content, b'{"detail":"Authentication credentials were not provided."}')

    def test_profile_destroy_my_profile_json(self):
        response = self.api_client.post(
            os.path.join(self.url_prefix, "login"),
            {"login": "miha", "password": "mihamiha", "return-type": "json"},
            format="json",
        )
        self.api_client.credentials(HTTP_AUTHORIZATION="sessionid " + response.data.get("sessionid", None))
        response = self.api_client.get("/account/profile/current", {"return-type": "json"}, format="json")
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        response = self.api_client.delete("/account/profile/current", {"return-type": "json"}, format="json")
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)
        response = self.api_client.get("/account/profile/current", {"return-type": "json"}, format="json")
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_logout_not_authorized(self):
        response = self.api_client.post(os.path.join(self.url_prefix, "logout"), {}, format="json")
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

    def test_logout(self):
        response = self.api_client.post(
            os.path.join(self.url_prefix, "login"), {"login": "miha", "password": "mihamiha"}, format="json"
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        response = self.api_client.post(os.path.join(self.url_prefix, "logout"), {}, format="json")
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestChangePasswordViewset(TestBase):
    url_prefix = "/account/"

    def setUp(self):
        super().setUp()
        self.api_client = APIClient()

    def test_change_password(self):
        response = self.api_client.post(
            os.path.join(self.url_prefix, "login"), {"login": "miha", "password": "mihamiha"}, format="json"
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        # Test to short password
        response = self.api_client.post(
            os.path.join(self.url_prefix, "change-password/"),
            {"old_password": "mihamiha", "password": "janez", "password_confirm": "janez"},
            format="json",
        )
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(
            response.content, b'{"password":["This password is too short. It must contain at least 8 characters."]}'
        )

        # Test ok password
        response = self.api_client.post(
            os.path.join(self.url_prefix, "change-password/"),
            {"old_password": "mihamiha", "password": "janezjanez", "password_confirm": "janezjanez"},
            format="json",
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        response = self.api_client.post(
            os.path.join(self.url_prefix, "login"), {"login": "miha", "password": "janezjanez"}, format="json"
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)


class TestSendResetPasswordLink(TestBase):
    def setUp(self):
        super().setUp()
        self.api_client = APIClient()

    def test_send_reset_password_link(self):
        response = self.api_client.post(
            os.path.join("/account/login"), {"login": "miha", "password": "mihamiha"}, format="json"
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        response = self.api_client.post("/account/send-reset-password-link/", {"email": "miha"}, format="json")
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        from django_project_base.settings import TEST_USER_ONE_DATA

        response = self.api_client.post(
            "/account/send-reset-password-link/", {"email": TEST_USER_ONE_DATA}, format="json"
        )
        self.assertEqual(response.status_code, status.HTTP_200_OK)



import datetime
from datetime import timedelta

import swapper
from django.core.management import call_command
from django.test import TestCase
from django.utils.timezone import make_aware


class MgmtCmdDeleteUsersTest(TestCase):
    def test_call_command(self):
        profile = swapper.load_model("django_project_base", "Profile").objects
        no_delete = profile.create(first_name="John", last_name="Doe", username="JohnDoe")
        delete_in_future = profile.create(
            first_name="John",
            last_name="Pitt",
            username="JohnPitt",
            delete_at=make_aware(datetime.datetime.now()) + timedelta(days=1),
        )
        delete_in_past = profile.create(
            first_name="Johanna",
            last_name="Pitt",
            username="JohannaPitt",
            delete_at=make_aware(datetime.datetime.now()) - timedelta(minutes=1),
        )
        call_command("delete_users")
        profiles = swapper.load_model("django_project_base", "Profile").objects.values_list("id", flat=True)
        self.assertIn(no_delete.id, profiles)
        self.assertIn(delete_in_future.id, profiles)
        self.assertNotIn(delete_in_past.id, profiles)



from django.core.cache import cache
from rest_framework import status
from rest_framework.test import APIClient

from django_project_base.settings import USER_CACHE_KEY
from example.demo_django_base.models import UserProfile
from tests.test_base import TestBase


class TestUsersCachingBackend(TestBase):
    def setUp(self):
        super().setUp()
        self.api_client = APIClient()

    def test_caching_for_bulk_update(self):
        self.assertTrue(self.api_client.login(username="miha", password="mihamiha"), "Not logged in")

        # Try to do something only superuser can
        response = self.api_client.put("/account/impersonate", {"username": "janez"}, format="json")
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

        self.assertIsNotNone(cache.get(USER_CACHE_KEY.format(id=1)))

        UserProfile.objects.filter(username__in=["miha", "janez"]).update(is_superuser=True, is_staff=True)

        # I still shouldn't be able to do superuser stuff
        response = self.api_client.put("/account/impersonate", {"username": "janez"}, format="json")
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)

        # Clearing cache, now Miha can do better stuff
        staff = UserProfile.objects.filter(is_staff=True)
        for user in staff:
            cache.delete(USER_CACHE_KEY.format(id=user.id))

        response = self.api_client.put("/account/impersonate", {"username": "janez"}, format="json")
        self.assertEqual(response.status_code, status.HTTP_200_OK)



# Generated by Django 4.2.4 on 2023-08-18 14:42
import copy

from django.conf import settings
import django.contrib.auth.models
import django.core.validators
from django.contrib.auth.hashers import make_password
from django.db import migrations, models
import django.db.models.deletion
import django_project_base.base.fields
import re
import taggit.managers

from django_project_base.settings import TEST_USER_ONE_DATA, TEST_USER_TWO_DATA


def forwards_func(apps, schema_editor):
    ProjectModel = apps.get_model("demo_django_base", "Project")
    UserProfileModel = apps.get_model("demo_django_base", "UserProfile")
    db_alias = schema_editor.connection.alias
    user_one_data: dict = copy.copy(TEST_USER_ONE_DATA)
    user_two_data: dict = copy.copy(TEST_USER_TWO_DATA)
    user_one_data.update(dict(password=make_password(TEST_USER_ONE_DATA["password"])))
    user_two_data.update(dict(password=make_password(TEST_USER_TWO_DATA["password"])))
    user1 = UserProfileModel.objects.using(db_alias).create(**user_one_data)
    user2 = UserProfileModel.objects.using(db_alias).create(**user_two_data)
    ProjectModel.objects.using(db_alias).bulk_create(
        [
            ProjectModel(name="Project One", slug="project-one", description="project one description", owner=user1),
            ProjectModel(name="Project Two", slug="project-two", description="project two description", owner=user2),
            ProjectModel(
                name="Project Three", slug="project-three", description="project three description", owner=user2
            ),
            ProjectModel(name="Project Four", slug="project-four", description="project four description", owner=user1),
        ]
    )


def reverse_func(apps, schema_editor):
    pass


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        ("auth", "0012_alter_user_first_name_max_length"),
    ]

    operations = [
        migrations.CreateModel(
            name="DemoProjectTag",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "name",
                    models.CharField(max_length=100, unique=True, verbose_name="name"),
                ),
                (
                    "slug",
                    models.SlugField(
                        allow_unicode=True,
                        max_length=100,
                        unique=True,
                        verbose_name="slug",
                    ),
                ),
                (
                    "color",
                    django_project_base.base.fields.HexColorField(max_length=7, verbose_name="Color"),
                ),
                (
                    "icon",
                    models.CharField(blank=True, max_length=10, null=True, verbose_name="Icon"),
                ),
                ("content", models.CharField(blank=True, max_length=20, null=True)),
            ],
            options={
                "verbose_name": "Tag",
                "verbose_name_plural": "Tags",
            },
        ),
        migrations.CreateModel(
            name="Permission",
            fields=[
                (
                    "permission_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="auth.permission",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=("auth.permission",),
            managers=[
                ("objects", django.contrib.auth.models.PermissionManager()),
            ],
        ),
        migrations.CreateModel(
            name="Project",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "name",
                    models.CharField(db_index=True, max_length=80, verbose_name="Name"),
                ),
                ("slug", models.SlugField(max_length=80, verbose_name="Slug")),
                (
                    "description",
                    models.TextField(blank=True, null=True, verbose_name="Description"),
                ),
                (
                    "logo",
                    models.FileField(blank=True, null=True, upload_to="", verbose_name="Logo"),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="Role",
            fields=[
                (
                    "group_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="auth.group",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=("auth.group",),
            managers=[
                ("objects", django.contrib.auth.models.GroupManager()),
            ],
        ),
        migrations.CreateModel(
            name="UserProfile",
            fields=[
                (
                    "user_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
                (
                    "bio",
                    models.TextField(blank=True, max_length=500, null=True, verbose_name="Bio"),
                ),
                (
                    "phone_number",
                    models.CharField(
                        blank=True,
                        max_length=20,
                        null=True,
                        verbose_name="Phone number",
                    ),
                ),
                (
                    "language",
                    models.CharField(blank=True, max_length=10, null=True, verbose_name="Language"),
                ),
                (
                    "theme",
                    models.CharField(blank=True, max_length=10, null=True, verbose_name="Theme"),
                ),
                (
                    "avatar",
                    models.FileField(blank=True, null=True, upload_to="", verbose_name="Avatar"),
                ),
                (
                    "reverse_full_name_order",
                    models.BooleanField(blank=True, null=True, verbose_name="Reverse full name order"),
                ),
                (
                    "delete_at",
                    models.DateTimeField(blank=True, null=True, verbose_name="Delete profile at"),
                ),
                ("password_invalid", models.BooleanField(default=False)),
            ],
            options={
                "abstract": False,
            },
            bases=("auth.user",),
            managers=[
                ("objects", django.contrib.auth.models.UserManager()),
            ],
        ),
        migrations.CreateModel(
            name="ProjectMember",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "member",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="projects",
                        to=settings.DJANGO_PROJECT_BASE_PROFILE_MODEL,
                        verbose_name="Owner",
                    ),
                ),
                (
                    "project",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="members",
                        to=settings.DJANGO_PROJECT_BASE_PROJECT_MODEL,
                        verbose_name="Project",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.AddField(
            model_name="project",
            name="owner",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="user_profiles",
                to=settings.DJANGO_PROJECT_BASE_PROFILE_MODEL,
            ),
        ),
        migrations.CreateModel(
            name="MergeUserGroup",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "users",
                    models.CharField(
                        max_length=1024,
                        validators=[
                            django.core.validators.RegexValidator(
                                re.compile("^\\d+(?:,\\d+)*\\Z"),
                                code="invalid",
                                message="Enter only digits separated by commas.",
                            )
                        ],
                    ),
                ),
                ("created_by", models.PositiveIntegerField()),
                (
                    "project",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.DJANGO_PROJECT_BASE_PROJECT_MODEL,
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="DpbTaggedItemThroughDemo",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "object_id",
                    models.IntegerField(db_index=True, verbose_name="object ID"),
                ),
                ("main_object", models.BooleanField(default=False)),
                (
                    "content_type",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="%(app_label)s_%(class)s_tagged_items",
                        to="contenttypes.contenttype",
                        verbose_name="content type",
                    ),
                ),
                (
                    "tag",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="%(app_label)s_%(class)s_items",
                        to=settings.DJANGO_PROJECT_BASE_TAG_MODEL,
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.AddField(
            model_name="demoprojecttag",
            name="project",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to=settings.DJANGO_PROJECT_BASE_PROJECT_MODEL,
                verbose_name="Project",
            ),
        ),
        migrations.CreateModel(
            name="Apartment",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("number", models.IntegerField()),
                (
                    "tags",
                    taggit.managers.TaggableManager(
                        help_text="A comma-separated list of tags.",
                        through="demo_django_base.DpbTaggedItemThroughDemo",
                        to="demo_django_base.DemoProjectTag",
                        verbose_name="Tags",
                    ),
                ),
            ],
        ),
        migrations.RunPython(forwards_func, reverse_func),
    ]



from django.test import TestCase

from django_project_base.settings import TEST_USER_ONE_DATA, TEST_USER_TWO_DATA


class TestBase(TestCase):
    def __login_with_user(self, user_data: dict) -> bool:
        return self.api_client.login(username=user_data["username"], password=user_data["password"])

    def _login_with_test_user_one(self) -> bool:
        return self.__login_with_user(TEST_USER_ONE_DATA)

    def _login_with_test_user_two(self) -> bool:
        return self.__login_with_user(TEST_USER_TWO_DATA)



"""
Django settings for demo_app project.

Generated by 'django-admin startproject' using Django 3.1.5.

For more information on this file, see
https://docs.djangoproject.com/en/3.1/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/3.1/ref/settings/
"""

import os
import sys

from pathlib import Path

from django_project_base import VERSION

# Build paths inside the project like this: BASE_DIR / 'subdir'.
from django_project_base.account.constants import ACCOUNT_APP_ID
from django_project_base.notifications import NOTIFICATIONS_APP_ID

try:
    from . import env
except:
    env = dict()

BASE_DIR = Path(__file__).resolve().parent.parent

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/3.1/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = "w0o6y0rwef0zijgd7m91w0b!p-(#l1zpna1%c1vvr7f17)x&*-"

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True
TESTING = len(sys.argv) > 1 and sys.argv[1] == "test"

ALLOWED_HOSTS = ["*"]

# Application definition

INSTALLED_APPS = [
    "django.contrib.contenttypes",
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "rest_framework",
    "rest_framework.authtoken",
    "rest_registration",
    "django_project_base",
    "example.demo_django_base",
    "drf_spectacular",
    NOTIFICATIONS_APP_ID,
    "social_django",
    ACCOUNT_APP_ID,
    "dynamicforms",
    "django_project_base.licensing",
]

if not getattr(env, "DEPLOY", True):
    INSTALLED_APPS.append("vue")

MIDDLEWARE = [
    "django_project_base.base.UrlVarsMiddleware",
    "django.middleware.security.SecurityMiddleware",
    # 'django.contrib.sessions.middleware.SessionMiddleware',
    "django_project_base.account.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.middleware.locale.LocaleMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "django_project_base.profiling.profile_middleware",
]

ROOT_URLCONF = "example.setup.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]

WSGI_APPLICATION = "example.setup.wsgi.application"

# Database
# https://docs.djangoproject.com/en/3.1/ref/settings/#databases

DATABASES = {
    "default": {
        "ENGINE": "django_project_base.query_tracker",
        "TRACKED_ENGINE": "django.db.backends.sqlite3",
        "NAME": os.path.join(BASE_DIR, "db.sqlite3"),
    }
}

# Password validation
# https://docs.djangoproject.com/en/3.1/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
    },
]

SOCIAL_AUTH_GOOGLE_OAUTH2_KEY = "*Client ID*"
SOCIAL_AUTH_GOOGLE_OAUTH2_SECRET = "*Client secret*"

SOCIAL_AUTH_GITHUB_KEY = "a1b2c3d4"
SOCIAL_AUTH_GITHUB_SECRET = "e5f6g7h8i9"

SOCIAL_AUTH_MICROSOFT_GRAPH_KEY = "..."
SOCIAL_AUTH_MICROSOFT_GRAPH_SECRET = "..."

# Internationalization
# https://docs.djangoproject.com/en/3.1/topics/i18n/

LANGUAGE_CODE = "en-us"
TIME_ZONE = "UTC"
USE_I18N = True
USE_L10N = True
USE_TZ = True

LOCALE_PATHS = [os.path.abspath(os.path.join(BASE_DIR, "../django_project_base/locale/"))]

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/3.1/howto/static-files/


STATIC_URL = "/static/"
STATICFILES_DIRS = (
    # os.path.join(BASE_DIR, "static/"),
    str(BASE_DIR).replace("example", "") + "django_project_base/static/",
)

DJANGO_PROJECT_BASE_PROJECT_MODEL = "demo_django_base.Project"
DJANGO_PROJECT_BASE_TAG_MODEL = "demo_django_base.DemoProjectTag"
DJANGO_PROJECT_BASE_PROFILE_MODEL = "demo_django_base.UserProfile"
DJANGO_PROJECT_BASE_PROJECTMEMBER_MODEL = "demo_django_base.ProjectMember"
DJANGO_PROJECT_BASE_MERGEUSERGROUP_MODEL = "demo_django_base.MergeUserGroup"
DJANGO_PROJECT_BASE_ROLE_MODEL = "demo_django_base.Role"
DJANGO_PROJECT_BASE_PERMISSION_MODEL = "demo_django_base.Permission"
DJANGO_PROJECT_BASE_PROJECTSETTINGS_MODEL = "demo_django_base.ProjectSettings"
DJANGO_PROJECT_BASE_INVITE_MODEL = "demo_django_base.ProjectInvite"

DJANGO_PROJECT_BASE_BASE_REQUEST_URL_VARIABLES = {
    "project": {
        "value_name": "current_project_slug",
        "url_part": (
            2,
            ("project", "account", "profile"),
        ),
    }
}

DEFAULT_FROM_EMAIL = "info@example.com"

REST_REGISTRATION = {
    "REGISTER_VERIFICATION_ENABLED": False,
    "REGISTER_EMAIL_VERIFICATION_ENABLED": False,
    "LOGIN_DEFAULT_SESSION_AUTHENTICATION_BACKEND": "django_project_base.base.auth_backends.UsersCachingBackend",
    "VERIFICATION_FROM_EMAIL": DEFAULT_FROM_EMAIL,
}

REST_FRAMEWORK = {
    "DEFAULT_SCHEMA_CLASS": "drf_spectacular.openapi.AutoSchema",
    "DEFAULT_RENDERER_CLASSES": (
        "rest_framework.renderers.JSONRenderer",
        "rest_framework.renderers.BrowsableAPIRenderer",
        "dynamicforms.renderers.ComponentDefRenderer",
    ),
    "DEFAULT_FILTER_BACKENDS": ("dynamicforms.filters.FilterBackend",),
}

SPECTACULAR_SETTINGS = {
    "TITLE": "Django Project Base Example API",
    "DESCRIPTION": "This is API documentation for Django project base example project. API is showcase for all"
    'available api endpoint for "Django project base" project',
    "VERSION": VERSION,
    "SWAGGER_UI_SETTINGS": {
        "deepLinking": True,
        "defaultModelsExpandDepth": 10,
        "defaultModelExpandDepth": 10,
        "tryItOutEnabled": True,
    },
    "COMPONENT_SPLIT_REQUEST": True,
}

AUTHENTICATION_BACKENDS = (
    "django_project_base.base.auth_backends.UsersCachingBackend",  # cache users for auth to gain performance
)

PROFILER_LONG_RUNNING_TASK_THRESHOLD = 1000

# Settings for Mailhog
# https://github.com/mailhog/MailHog
# MailHog is an email testing tool for developers. Configure your application to use MailHog for SMTP delivery.
EMAIL_BACKEND = "django.core.mail.backends.smtp.EmailBackend"
EMAIL_HOST = "127.0.0.1"
EMAIL_PORT = 1025
EMAIL_USE_SSL = False
EMAIL_HOST_PASSWORD = ""
SERVER_EMAIL = ""

MEDIA_URL = "/media/"
MEDIA_ROOT = os.path.join(BASE_DIR, "media")

DYNAMICFORMS = {
    "allow_anonymous_user_to_preupload_files": True,
}

CACHES = {
    "default": {
        "BACKEND": "django.core.cache.backends.locmem.LocMemCache",
        "LOCATION": "cache",
    }
}

DEFAULT_AUTO_FIELD = "django.db.models.AutoField"



import os

from celery import bootsteps, Celery
from click import Option
from kombu import Exchange, Queue
from kombu.entity import TRANSIENT_DELIVERY_MODE

from django_project_base.celery.settings import NOTIFICATIONS_QUEUE_VISIBILITY_TIMEOUT
from django_project_base.constants import NOTIFICATION_QUEUE_NAME

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "setup.settings")

app = Celery(
    "django_project_base",
    include=[
        "django_project_base.celery.background_tasks.notification_tasks",
    ],
)
app.config_from_object("django.conf:settings", namespace="CELERY")

app.conf.task_queues = [
    Queue(
        NOTIFICATION_QUEUE_NAME,
        Exchange("transient", delivery_mode=TRANSIENT_DELIVERY_MODE),
        routing_key=NOTIFICATION_QUEUE_NAME,
        durable=False,
    ),
]

app.conf.beat_schedule = {
    "scheduler": {
        "task": "background_tasks.notification_tasks.notification_scheduler_task",
        "schedule": 3,
        "options": {"queue": "notification"},
    },
}

app.conf.task_ignore_result = True
app.conf.worker_send_task_events = False
app.conf.broker_transport_options = {"visibility_timeout": NOTIFICATIONS_QUEUE_VISIBILITY_TIMEOUT}
setting_option = Option(("--settings",), is_flag=False, help="Django settings file path", default="")
app.user_options["worker"].add(setting_option)
app.user_options["beat"].add(setting_option)


class CeleryBootstep(bootsteps.Step):
    def __init__(self, parent, **options):
        super().__init__(parent, **options)
        app.conf.setdefault("django-settings-module", options.get("settings", ""))


app.steps["worker"].add(CeleryBootstep)
app.steps["beat"].add(CeleryBootstep)

# RUN WORKER AS
# celery -A django_project_base.celery.celery worker -l INFO -Q notification --concurrency=1



from enum import Enum


class LicenseType(Enum):
    EMAIL = "email"
    SMS = "sms"


class LicenseTypeData:
    price: float

    def __init__(self, price: float) -> None:
        self.price = price



from django.shortcuts import render


# Create your views here.
def index_view(request):
    return render(request=request, template_name="index.html")


def page1_view(request):
    return render(request=request, template_name="page1.html")



from rest_framework import status
from rest_framework.test import APIClient

from django_project_base.base.auth_backends import user_cache_invalidate
from example.demo_django_base.models import UserProfile
from tests.test_base import TestBase


class TestImpersonateUserViewset(TestBase):
    def setUp(self):
        super().setUp()
        self.api_client = APIClient()

    def test_impersonate(self):
        self.assertTrue(self._login_with_test_user_two(), "Not logged in")

        # Janez is not superuser and is not allowed to impersonate
        response = self.api_client.put("/account/impersonate", {"email": "user1@user1.si"}, format="json")
        self.assertEqual(response.status_code, 403)

        # make Janez a superuser
        janez = UserProfile.objects.get(username="janez")
        janez.is_staff = True
        janez.is_superuser = True
        janez.save()
        # delete user cache
        user_cache_invalidate(janez)

        # Now Janez should be able to impersonate
        response = self.api_client.put("/account/impersonate", {"email": "user1@user1.si"}, format="json")
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        response = self.api_client.delete("/account/impersonate", {}, format="json")
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_impersonate_with_username(self):
        self.assertTrue(self._login_with_test_user_two(), "Not logged in")

        # make Janez a superuser
        janez = UserProfile.objects.get(username="janez")
        janez.is_staff = True
        janez.is_superuser = True
        janez.save()
        # delete user cache
        user_cache_invalidate(janez)

        response = self.api_client.put("/account/impersonate", {"username": "miha"}, format="json")
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        response = self.api_client.delete("/account/impersonate", {}, format="json")
        self.assertEqual(response.status_code, status.HTTP_200_OK)

    def test_impersonate_with_id(self):
        self.assertTrue(self._login_with_test_user_two(), "Not logged in")

        # make Janez a superuser
        janez = UserProfile.objects.get(username="janez")
        janez.is_staff = True
        janez.is_superuser = True
        janez.save()
        # delete user cache
        user_cache_invalidate(janez)

        response = self.api_client.put("/account/impersonate", {"id": 1}, format="json")
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        response = self.api_client.delete("/account/impersonate", {}, format="json")
        self.assertEqual(response.status_code, status.HTTP_200_OK)



import swapper
from django.db.models import Model
from rest_framework import status
from rest_framework.response import Response
from rest_framework.test import APIClient

from django_project_base.rest.project_role import ProjectRole
from django_project_base.settings import TEST_USER_ONE_DATA
from example.demo_django_base.models import UserProfile
from tests.test_base import TestBase


class TestRole(TestBase):
    url = "/project-role"
    project: Model
    test_role_name = "test-role"

    def setUp(self):
        super().setUp()
        self.api_client = APIClient()
        self.project: Model = swapper.load_model("django_project_base", "Project").objects.create(
            name="test-project",
            owner=UserProfile.objects.get(username=TEST_USER_ONE_DATA["username"]),
            slug="test-project-slug",
        )
        response = self.api_client.post(
            "/account/login",
            {
                "login": TEST_USER_ONE_DATA["username"],
                "password": TEST_USER_ONE_DATA["password"],
                "return-type": "json",
            },
            format="json",
        )
        self.api_client.credentials(HTTP_AUTHORIZATION="sessionid " + response.data.get("sessionid", None))

    def __create_role(self, payload: dict = {}) -> Response:
        return self.api_client.post(
            self.url,
            {**{"name": self.test_role_name, "project": self.project.pk}, **payload},
        )

    def test_create_project_role(self):
        role_class = swapper.load_model("django_project_base", "Role")

        response: Response = self.__create_role()
        self.assertEqual(status.HTTP_201_CREATED, response.status_code)
        project_role: dict = response.json()
        self.assertEqual(project_role.get("name"), self.test_role_name)
        group = role_class.objects.get(pk=project_role["id"])
        self.assertEqual(
            group.name,
            f'{self.project.pk}{ProjectRole.delimiter}{project_role.get("name")}',
        )

    def test_list_project_role(self):
        self.__create_role()
        list_response: Response = self.api_client.get(f"{self.url}?project={self.project.pk}")
        self.assertEqual(list_response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(list_response.json()), 1)
        self.assertEqual(list_response.json()[0].get("name"), self.test_role_name)

    def test_retrieve_project_role(self):
        role_pk: int = self.__create_role().json().get("id")
        retrieve_response: Response = self.api_client.get(f"{self.url}/{role_pk}")
        self.assertEqual(retrieve_response.status_code, status.HTTP_200_OK)
        self.assertEqual(retrieve_response.json().get("id"), role_pk)
        self.assertEqual(retrieve_response.json().get("name"), self.test_role_name)

    def __test_update_role(self, action_method):
        role_pk: int = self.__create_role().json().get("id")
        update_response: Response = getattr(self.api_client, action_method.lower())(
            f"{self.url}/{role_pk}", {"name": "new-role-name"}
        )
        self.assertEqual(update_response.status_code, status.HTTP_200_OK)
        self.assertEqual(update_response.json().get("name"), "new-role-name")
        self.assertEqual(
            self.api_client.get(f"{self.url}/{role_pk}").json().get("name"),
            "new-role-name",
        )

    def test_update_project_role(self):
        self.__test_update_role("put")

    def test_partial_update_project_role(self):
        self.__test_update_role("patch")

    def test_delete_project_role(self):
        role_pk: int = self.__create_role().json().get("id")
        delete_response: Response = self.api_client.delete(f"{self.url}/{role_pk}")
        self.assertEqual(delete_response.status_code, status.HTTP_204_NO_CONTENT)
        self.assertEqual(0, len(self.api_client.get(f"{self.url}?project={self.project.pk}").json()))



# Generated by Django 4.2.4 on 2023-09-20 08:16

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("demo_django_base", "0005_remove_project_email_sender_id_and_more"),
    ]

    operations = [
        migrations.AlterField(
            model_name="projectsettings",
            name="value_type",
            field=models.CharField(
                choices=[
                    ("integer", "Whole number"),
                    ("float", "Decimal number"),
                    ("bool", "True/False"),
                    ("char", "String"),
                    ("custom", "Custom"),
                ],
                max_length=10,
            ),
        ),
    ]



import datetime
import time

from typing import Optional

from django.core.cache import cache

from django_project_base.celery.background_tasks.base_task import BaseTask
from django_project_base.celery.celery import app
from django_project_base.celery.settings import NOTIFICATION_QUEABLE_HARD_TIME_LIMIT, NOTIFICATION_SEND_PAUSE_SECONDS
from django_project_base.constants import NOTIFICATION_QUEUE_NAME
from django_project_base.notifications.base.send_notification_service import SendNotificationService
from django_project_base.notifications.models import DjangoProjectBaseNotification
from django_project_base.serialization import CacheLock

LAST_MAIL_SENT_CK = "last-notification-was-sent-timestamp"


class SendNotificationTask(BaseTask):
    name = "background_tasks.notification_tasks.send_notification_task"

    max_retries = 0
    time_limit = NOTIFICATION_QUEABLE_HARD_TIME_LIMIT
    soft_time_limit = NOTIFICATION_QUEABLE_HARD_TIME_LIMIT - 3
    default_retry_delay = 0

    def run(self, notification: "DjangoProjectBaseNotification", extra_data):  # noqa: F821
        with CacheLock(f"notification_send_task_{notification.id}", timeout=-1) as cl:
            cl()
            if DjangoProjectBaseNotification.objects.filter(id=notification.id, done=True).exists():
                return
            try:
                last_sent: Optional[float] = cache.get(LAST_MAIL_SENT_CK)
                time_from_last_sent: float = time.time() - last_sent if last_sent else 0
                if time_from_last_sent < NOTIFICATION_SEND_PAUSE_SECONDS:
                    time.sleep(int(NOTIFICATION_SEND_PAUSE_SECONDS - time_from_last_sent))
                SendNotificationService(settings=self.settings).make_send(
                    notification=notification, extra_data=extra_data
                )
            finally:
                cache.set(LAST_MAIL_SENT_CK, time.time(), timeout=NOTIFICATION_SEND_PAUSE_SECONDS + 1)
        self._close_db()


send_notification_task = app.register_task(SendNotificationTask())


class NotificationSchedulerTask(BaseTask):
    name = "background_tasks.notification_tasks.notification_scheduler_task"

    def run(self):
        from django_project_base.notifications.base.enums import NotificationType

        if getattr(self.settings, "TESTING", False):
            return

        with CacheLock("notification_scheduler_task", timeout=-1) as cl:
            cl()
            for notification in DjangoProjectBaseNotification.objects.filter(
                done=False,
                delayed_to__lte=int(datetime.datetime.now().timestamp()),
                type=NotificationType.STANDARD.value,
            ):
                notification.prepare_for_send()
                send_notification_task.apply_async(
                    (notification, notification.extra_data), queue=NOTIFICATION_QUEUE_NAME, serializer="pickle"
                )

        self._close_db()


notification_scheduler_task = app.register_task(NotificationSchedulerTask())



from typing import List

from django.conf import Settings
from django.utils.translation import gettext_lazy as _

from django_project_base.licensing.license_type_data import LicenseType
from django_project_base.notifications.base.channels.channel import Channel, Recipient
from django_project_base.notifications.base.enums import ChannelIdentifier
from django_project_base.notifications.models import DjangoProjectBaseNotification


class SmsChannel(Channel):

    @staticmethod
    def get_license_type():
        return LicenseType.SMS

    id = ChannelIdentifier.SMS.value

    name = "SMS"

    provider_setting_name = "NOTIFICATIONS_SMS_PROVIDER"

    @property
    def verbose_name(self):
        return _("SMS")

    def get_recipients(
        self, notification: DjangoProjectBaseNotification, unique_identifier="", phone_number_validator=None
    ):
        return list(
            set(
                super().get_recipients(
                    notification, unique_identifier="phone_number", phone_number_validator=phone_number_validator
                )
            )
        )

    def send(self, notification: DjangoProjectBaseNotification, extra_data: dict, settings: Settings, **kwargs) -> int:  # noqa: F821
        return super().send(notification=notification, extra_data=extra_data, settings=settings)

    def clean_sms_recipients(self, recipients: List[Recipient]) -> List[Recipient]:
        return list(filter(lambda r: r.phone_number and len(r.phone_number), self.clean_recipients(recipients)))



# Generated by Django 4.2.4 on 2023-10-05 07:54
from gettext import gettext

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import uuid

from django_project_base.constants import INVITE_NOTIFICATION_TEXT


def forwards_func(apps, schema_editor):
    project_sett = apps.get_model("demo_django_base", "ProjectSettings")
    for project in apps.get_model("demo_django_base", "Project").objects.all():
        if not project_sett.objects.filter(
            project=project,
            name=INVITE_NOTIFICATION_TEXT,
        ).exists():
            project_sett.objects.create(
                project=project,
                name=INVITE_NOTIFICATION_TEXT,
                description=gettext(
                    "Text for invite notification view. __LINK__ is url to invite confirmation placeholder."
                ),
                value=gettext("Click link to join project: __LINK__"),
                value_type="char",
            )


def reverse_func(apps, schema_editor):
    pass


class Migration(migrations.Migration):
    dependencies = [
        ("demo_django_base", "0007_merge_20230927_0554"),
    ]

    operations = [
        migrations.CreateModel(
            name="ProjectInvite",
            fields=[
                (
                    "id",
                    models.UUIDField(
                        default=uuid.uuid4,
                        primary_key=True,
                        serialize=False,
                        verbose_name="Id",
                    ),
                ),
                ("email", models.CharField(max_length=255, verbose_name="eMail")),
                (
                    "text",
                    models.TextField(null=True, verbose_name="Invitation message"),
                ),
                ("accepted", models.DateTimeField(blank=True, null=True)),
                (
                    "invited_by",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="project_user_invites",
                        to=settings.DJANGO_PROJECT_BASE_PROFILE_MODEL,
                    ),
                ),
                (
                    "project",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.DJANGO_PROJECT_BASE_PROJECT_MODEL,
                    ),
                ),
                (
                    "role",
                    models.ForeignKey(
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to=settings.DJANGO_PROJECT_BASE_ROLE_MODEL,
                    ),
                ),
            ],
            options={
                "abstract": False,
                "unique_together": {("project", "email")},
            },
        ),
        migrations.RunPython(forwards_func, reverse_func),
    ]



# Generated by Django 5.1.7 on 2025-11-04 12:50

import django.db.models.deletion

from django.db import migrations, models


def update_notifications_done(apps, schema_editor):
    from django_project_base.notifications.models import DjangoProjectBaseNotification

    apps.get_model("notifications", "DjangoProjectBaseNotification").objects.exclude(
        delayed_to=DjangoProjectBaseNotification.DELAYED_INDEFINITELY
    ).update(done=True)


class Migration(migrations.Migration):
    dependencies = [
        ("notifications", "0011_djangoprojectbasemessageattachment"),
    ]

    operations = [
        migrations.AddField(
            model_name="djangoprojectbasenotification",
            name="done",
            field=models.BooleanField(default=False),
        ),
        migrations.AlterField(
            model_name="djangoprojectbasemessageattachment",
            name="message",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="attachments",
                to="notifications.djangoprojectbasemessage",
                verbose_name="Message",
            ),
        ),
        migrations.AddIndex(
            model_name="djangoprojectbasenotification",
            index=models.Index(fields=["delayed_to", "done"], name="notificatio_delayed_28febf_idx"),
        ),
        migrations.RunPython(update_notifications_done),
    ]



import swapper
from django.db.models import Model
from django.utils.crypto import get_random_string
from rest_framework import status
from rest_framework.response import Response
from rest_framework.test import APIClient

from django_project_base.rest.project import ProjectSerializer
from django_project_base.settings import TEST_USER_ONE_DATA
from example.demo_django_base.models import UserProfile
from tests.test_base import TestBase


class TestProject(TestBase):
    url: str = "/project"

    def setUp(self):
        super().setUp()
        self.api_client = APIClient()
        self._login_with_test_user_one()
        self._create_project()

    def _create_project(self, payload: dict = {}) -> Response:
        _payload: dict = {
            "name": "test-project",
            "owner": UserProfile.objects.get(username=TEST_USER_ONE_DATA["username"]).pk,
            "slug": get_random_string(length=8),
        }
        _payload.update(payload)
        return self.api_client.post(self.url, _payload)

    def test_create_project(self):
        self.assertEqual(status.HTTP_201_CREATED, self._create_project().status_code)

    def test_list_project(self):
        list_response: Response = self.api_client.get(self.url)
        self.assertEqual(3, len(list_response.data))

    def test_retrieve_project(self):
        retrieve_project_pk: Response = self.api_client.get(f"{self.url}/1")
        self.assertEqual(status.HTTP_200_OK, retrieve_project_pk.status_code)
        retrieve_project_slug: Response = self.api_client.get(
            f'{self.url}/project-{retrieve_project_pk.data["slug"]}',
            HTTP_CURRENT_PROJECT=swapper.load_model("django_project_base", "Project").objects.first().slug,
        )
        self.assertEqual(
            retrieve_project_pk.data[ProjectSerializer.Meta.model._meta.pk.name],
            retrieve_project_slug.data[ProjectSerializer.Meta.model._meta.pk.name],
        )

    def test_update_project(self):
        project: Model = ProjectSerializer.Meta.model.objects.last()

        update_project: Response = self.api_client.patch(
            f"{self.url}/{project.pk}",
            {"name": "updated-name"},
            HTTP_CURRENT_PROJECT=project.slug
        )
        self.assertEqual(update_project.status_code, status.HTTP_200_OK)
        project.refresh_from_db()
        self.assertEqual(project.name, "updated-name")



# Generated by Django 4.2.4 on 2023-09-05 08:49

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion

from django_project_base.constants import (
    EMAIL_SENDER_ID_SETTING_NAME,
    SMS_SENDER_ID_SETTING_NAME,
    NOTIFY_NEW_USER_SETTING_NAME,
)


def forwards_func(apps, schema_editor):
    ProjectModel = apps.get_model("demo_django_base", "Project")
    ProjectModelSettings = apps.get_model("demo_django_base", "ProjectSettings")
    for project in ProjectModel.objects.all():
        email_sender = project.email_sender_id
        sms_sender = project.sms_sender_id

        ProjectModelSettings.objects.create(
            name=EMAIL_SENDER_ID_SETTING_NAME,
            project=project,
            description="From email address for notifications",
            value=email_sender or "",
            value_type="char",
        )

        ProjectModelSettings.objects.create(
            project=project,
            name=SMS_SENDER_ID_SETTING_NAME,
            description="Sms sender name for notifications",
            value=sms_sender or "",
            value_type="char",
        )

        ProjectModelSettings.objects.create(
            project=project,
            name=NOTIFY_NEW_USER_SETTING_NAME,
            description="Notify new user account was created for him",
            value=True,
            value_type="bool",
        )


def reverse_func(apps, schema_editor):
    pass


class Migration(migrations.Migration):
    dependencies = [
        (
            "demo_django_base",
            "0004_alter_demoprojecttag_name_alter_demoprojecttag_slug_and_more",
        ),
    ]

    operations = [
        migrations.CreateModel(
            name="ProjectSettings",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "name",
                    models.CharField(db_index=True, max_length=80, verbose_name="Name"),
                ),
                (
                    "description",
                    models.CharField(max_length=120, verbose_name="Description"),
                ),
                ("value", models.CharField(max_length=320, verbose_name="Value")),
                (
                    "value_type",
                    models.CharField(
                        choices=[
                            ("integer", "Whole number"),
                            ("float", "Decimal number"),
                            ("bool", "true/false"),
                            ("char", "String"),
                        ],
                        max_length=10,
                    ),
                ),
                (
                    "project",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to=settings.DJANGO_PROJECT_BASE_PROJECT_MODEL,
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
        migrations.RunPython(forwards_func, reverse_func),
        migrations.RemoveField(
            model_name="project",
            name="email_sender_id",
        ),
        migrations.RemoveField(
            model_name="project",
            name="sms_sender_id",
        ),
        migrations.AlterUniqueTogether(
            name="projectsettings",
            unique_together={("project", "name")},
        ),
    ]



# Generated by Django 5.1.7 on 2025-11-04 12:50

import swapper

from django.contrib.auth import get_user_model
from django.db import migrations


def delete_dlr_settings(apps, schema_editor):
    ProjectModelSettings = swapper.load_model("django_project_base", "ProjectSettings")
    ProjectModelSettings._base_manager.filter(
        name__in=[
            "aws-sns-sms-dlr-user",
            "aws-sns-sms-dlr-password",
            "aws-ses-email-dlr-user",
            "aws-ses-email-dlr-password",
            "nexmo-sms-dlr-user",
            "nexmo-sms-dlr-password",
            "t2-sms-dlr-user",
            "t2-sms-dlr-password",
        ]
    ).delete()

    get_user_model().objects.filter(
        username__istartswith="t2-", username__iendswith="-username", email="klemen.spruk@velis.si"
    ).update(email="t2-sms_dlr@velis.si")


class Migration(migrations.Migration):
    dependencies = [
        ("notifications", "0012_djangoprojectbasenotification_done_and_more"),
    ]

    operations = [
        migrations.RunPython(delete_dlr_settings),
    ]



import datetime

import requests

from rest_framework import status

from django_project_base.country_holidays import holidays_api_url
from tests.test_base import TestBase


class RetrieveHolidaysTest(TestBase):
    def test_retrieve_holidays(self):
        response: requests.Response = requests.get(holidays_api_url % (datetime.datetime.now().year, "SI"), timeout=6)
        self.assertEqual(response.status_code, status.HTTP_200_OK)



# Generated by Django 4.2.4 on 2023-09-01 10:59

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("demo_django_base", "0003_project_email_sender_id_project_sms_sender_id"),
    ]

    operations = [
        migrations.AlterField(
            model_name="demoprojecttag",
            name="name",
            field=models.CharField(max_length=100, verbose_name="name"),
        ),
        migrations.AlterField(
            model_name="demoprojecttag",
            name="slug",
            field=models.SlugField(allow_unicode=True, max_length=100, verbose_name="slug"),
        ),
        migrations.AlterUniqueTogether(
            name="demoprojecttag",
            unique_together={("project", "slug"), ("project", "name")},
        ),
    ]



"""demo_app URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/3.1/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""
from django.conf import settings
from django.conf.urls.static import static
from django.contrib import admin
from django.urls import include, path, re_path
from django.views.i18n import JavaScriptCatalog
from drf_spectacular.views import SpectacularAPIView, SpectacularSwaggerView

from django_project_base.notifications.rest.router import notifications_router
from django_project_base.profiling import app_debug_view
from django_project_base.settings import DOCUMENTATION_DIRECTORY
from django_project_base.views import documentation_view
from example.demo_django_base.views import index_view, page1_view

urlpatterns = [
    path("", index_view, name="index"),
    path("page1/", page1_view, name="page1"),
    path("dpb_admin/", admin.site.urls),
    path("schema/", SpectacularAPIView.as_view(), name="schema"),
    path(
        "schema/swagger-ui/",
        SpectacularSwaggerView.as_view(
            url_name="schema",
        ),
        name="swagger-ui",
    ),
    path("account/", include("django_project_base.account.urls")),
    path("", include("django_project_base.licensing.urls")),
    path("", include(notifications_router.urls)),
    path("", include("django_project_base.urls")),
    path("app-debug/", app_debug_view, name="app-debug"),
    re_path(
        r"^docs-files/(?P<path>.*)$", documentation_view, {"document_root": DOCUMENTATION_DIRECTORY}, name="docs-files"
    ),
    path("account/social/", include("social_django.urls", namespace="social")),
    path("jsi18n/", JavaScriptCatalog.as_view(), name="javascript-catalog"),
    re_path(r"^dynamicforms/", include("dynamicforms.urls")),
] + static(settings.STATIC_URL, document_root=settings.STATIC_ROOT)



from django.conf import settings
from rest_framework import status
from rest_framework.test import APIClient

from example.demo_django_base.models import Project, UserProfile
from tests.test_base import TestBase


class TestProject(TestBase):
    def setUp(self):
        super().setUp()
        self.api_client = APIClient()

    def test_construction(self):
        assert Project()


class TestProfile(TestBase):
    def setUp(self):
        super().setUp()
        self.api_client = APIClient()

    def test_construction(self):
        assert UserProfile()

    def test_reverse_full_name_order(self):
        self.assertTrue(self._login_with_test_user_two(), "Not logged in")
        response = self.api_client.get("/account/profile/current", {}, format="json")
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data["full_name"], "Janez Novak")

        settings.PROFILE_REVERSE_FULL_NAME_ORDER = True
        response = self.api_client.get("/account/profile/current", {}, format="json")
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data["full_name"], "Novak Janez")

        settings.PROFILE_REVERSE_FULL_NAME_ORDER = False
        janez = UserProfile.objects.get(username="janez")
        janez.reverse_full_name_order = True
        janez.save()
        response = self.api_client.get("/account/profile/current", {}, format="json")
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data["full_name"], "Novak Janez")



# Generated by Django 4.2.4 on 2023-09-27 05:54

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("demo_django_base", "0006_alter_projectsettings_value_type"),
        ("demo_django_base", "0006_projectsettings_reserved_and_more"),
    ]

    operations = []
